@@ -1,6 +1,6 @@
 <?php
 
-require_once("java/Java.inc");
+if(!extension_loaded('java')) require_once("java/Java.inc");
 
 $session = java_session();
 ?>
